# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "PlanSetPricingParams",
    "MinimumSpend",
    "MinimumSpendMinimum",
    "OveragePricingModel",
    "OveragePricingModelPricePeriod",
    "OveragePricingModelPricePeriodCreditRate",
    "OveragePricingModelPricePeriodPrice",
    "OveragePricingModelPricePeriodTier",
    "OveragePricingModelPricePeriodTierFlatPrice",
    "OveragePricingModelPricePeriodTierUnitPrice",
    "OveragePricingModelEntitlement",
    "OveragePricingModelEntitlementMonthlyResetPeriodConfiguration",
    "OveragePricingModelEntitlementWeeklyResetPeriodConfiguration",
    "OveragePricingModelEntitlementYearlyResetPeriodConfiguration",
    "PricingModel",
    "PricingModelPricePeriod",
    "PricingModelPricePeriodCreditRate",
    "PricingModelPricePeriodPrice",
    "PricingModelPricePeriodTier",
    "PricingModelPricePeriodTierFlatPrice",
    "PricingModelPricePeriodTierUnitPrice",
    "PricingModelMonthlyResetPeriodConfiguration",
    "PricingModelWeeklyResetPeriodConfiguration",
    "PricingModelYearlyResetPeriodConfiguration",
]


class PlanSetPricingParams(TypedDict, total=False):
    pricing_type: Required[Annotated[Literal["FREE", "PAID", "CUSTOM"], PropertyInfo(alias="pricingType")]]
    """The pricing type (FREE, PAID, or CUSTOM)"""

    billing_id: Annotated[str, PropertyInfo(alias="billingId")]
    """Deprecated: billing integration ID"""

    minimum_spend: Annotated[Optional[Iterable[MinimumSpend]], PropertyInfo(alias="minimumSpend")]
    """Minimum spend configuration per billing period"""

    overage_billing_period: Annotated[
        Literal["ON_SUBSCRIPTION_RENEWAL", "MONTHLY"], PropertyInfo(alias="overageBillingPeriod")
    ]
    """When overage charges are billed"""

    overage_pricing_models: Annotated[Iterable[OveragePricingModel], PropertyInfo(alias="overagePricingModels")]
    """Array of overage pricing model configurations"""

    pricing_models: Annotated[Iterable[PricingModel], PropertyInfo(alias="pricingModels")]
    """Array of pricing model configurations"""


class MinimumSpendMinimum(TypedDict, total=False):
    """The minimum spend amount"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class MinimumSpend(TypedDict, total=False):
    """Minimum spend configuration for a billing period."""

    billing_period: Required[Annotated[Literal["MONTHLY", "ANNUALLY"], PropertyInfo(alias="billingPeriod")]]
    """The billing period"""

    minimum: Required[MinimumSpendMinimum]
    """The minimum spend amount"""


class OveragePricingModelPricePeriodCreditRate(TypedDict, total=False):
    """Credit rate configuration for credit-based pricing"""

    amount: Required[float]
    """The credit rate amount"""

    currency_id: Required[Annotated[str, PropertyInfo(alias="currencyId")]]
    """The custom currency ID"""

    cost_formula: Annotated[str, PropertyInfo(alias="costFormula")]
    """Optional cost formula expression"""


class OveragePricingModelPricePeriodPrice(TypedDict, total=False):
    """The price amount and currency"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class OveragePricingModelPricePeriodTierFlatPrice(TypedDict, total=False):
    """Flat price for this tier"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class OveragePricingModelPricePeriodTierUnitPrice(TypedDict, total=False):
    """Per-unit price in this tier"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class OveragePricingModelPricePeriodTier(TypedDict, total=False):
    """A tier in tiered pricing."""

    flat_price: Annotated[OveragePricingModelPricePeriodTierFlatPrice, PropertyInfo(alias="flatPrice")]
    """Flat price for this tier"""

    unit_price: Annotated[OveragePricingModelPricePeriodTierUnitPrice, PropertyInfo(alias="unitPrice")]
    """Per-unit price in this tier"""

    up_to: Annotated[float, PropertyInfo(alias="upTo")]
    """Upper bound of this tier (null for unlimited)"""


class OveragePricingModelPricePeriod(TypedDict, total=False):
    """Price configuration for a specific billing period."""

    billing_period: Required[Annotated[Literal["MONTHLY", "ANNUALLY"], PropertyInfo(alias="billingPeriod")]]
    """The billing period (MONTHLY or ANNUALLY)"""

    billing_country_code: Annotated[str, PropertyInfo(alias="billingCountryCode")]
    """ISO country code for localized pricing"""

    block_size: Annotated[float, PropertyInfo(alias="blockSize")]
    """Block size for usage-based pricing"""

    credit_grant_cadence: Annotated[
        Literal["BEGINNING_OF_BILLING_PERIOD", "MONTHLY"], PropertyInfo(alias="creditGrantCadence")
    ]
    """When credits are granted"""

    credit_rate: Annotated[OveragePricingModelPricePeriodCreditRate, PropertyInfo(alias="creditRate")]
    """Credit rate configuration for credit-based pricing"""

    price: OveragePricingModelPricePeriodPrice
    """The price amount and currency"""

    tiers: Iterable[OveragePricingModelPricePeriodTier]
    """Tiered pricing configuration"""


class OveragePricingModelEntitlementMonthlyResetPeriodConfiguration(TypedDict, total=False):
    """Monthly reset configuration"""

    according_to: Required[
        Annotated[Literal["SubscriptionStart", "StartOfTheMonth"], PropertyInfo(alias="accordingTo")]
    ]
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class OveragePricingModelEntitlementWeeklyResetPeriodConfiguration(TypedDict, total=False):
    """Weekly reset configuration"""

    according_to: Required[
        Annotated[
            Literal[
                "SubscriptionStart",
                "EverySunday",
                "EveryMonday",
                "EveryTuesday",
                "EveryWednesday",
                "EveryThursday",
                "EveryFriday",
                "EverySaturday",
            ],
            PropertyInfo(alias="accordingTo"),
        ]
    ]
    """Reset anchor (SubscriptionStart or specific day)"""


class OveragePricingModelEntitlementYearlyResetPeriodConfiguration(TypedDict, total=False):
    """Yearly reset configuration"""

    according_to: Required[Annotated[Literal["SubscriptionStart"], PropertyInfo(alias="accordingTo")]]
    """Reset anchor (SubscriptionStart)"""


class OveragePricingModelEntitlement(TypedDict, total=False):
    """Entitlement configuration for the overage feature"""

    feature_id: Required[Annotated[str, PropertyInfo(alias="featureId")]]
    """The feature ID for the entitlement"""

    has_soft_limit: Annotated[bool, PropertyInfo(alias="hasSoftLimit")]
    """Whether the limit is soft (allows overage)"""

    has_unlimited_usage: Annotated[bool, PropertyInfo(alias="hasUnlimitedUsage")]
    """Whether usage is unlimited"""

    monthly_reset_period_configuration: Annotated[
        OveragePricingModelEntitlementMonthlyResetPeriodConfiguration,
        PropertyInfo(alias="monthlyResetPeriodConfiguration"),
    ]
    """Monthly reset configuration"""

    reset_period: Annotated[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"], PropertyInfo(alias="resetPeriod")]
    """The usage reset period"""

    usage_limit: Annotated[float, PropertyInfo(alias="usageLimit")]
    """The usage limit before overage kicks in"""

    weekly_reset_period_configuration: Annotated[
        OveragePricingModelEntitlementWeeklyResetPeriodConfiguration,
        PropertyInfo(alias="weeklyResetPeriodConfiguration"),
    ]
    """Weekly reset configuration"""

    yearly_reset_period_configuration: Annotated[
        OveragePricingModelEntitlementYearlyResetPeriodConfiguration,
        PropertyInfo(alias="yearlyResetPeriodConfiguration"),
    ]
    """Yearly reset configuration"""


class OveragePricingModel(TypedDict, total=False):
    """Overage pricing model configuration."""

    billing_model: Required[
        Annotated[
            Literal["FLAT_FEE", "MINIMUM_SPEND", "PER_UNIT", "USAGE_BASED", "CREDIT_BASED"],
            PropertyInfo(alias="billingModel"),
        ]
    ]
    """The billing model for overages"""

    price_periods: Required[Annotated[Iterable[OveragePricingModelPricePeriod], PropertyInfo(alias="pricePeriods")]]
    """Price periods for overage pricing"""

    billing_cadence: Annotated[Literal["RECURRING", "ONE_OFF"], PropertyInfo(alias="billingCadence")]
    """The billing cadence for overages"""

    entitlement: OveragePricingModelEntitlement
    """Entitlement configuration for the overage feature"""

    feature_id: Annotated[str, PropertyInfo(alias="featureId")]
    """The feature ID for overage pricing"""

    top_up_custom_currency_id: Annotated[str, PropertyInfo(alias="topUpCustomCurrencyId")]
    """Custom currency ID for overage top-up"""


class PricingModelPricePeriodCreditRate(TypedDict, total=False):
    """Credit rate configuration for credit-based pricing"""

    amount: Required[float]
    """The credit rate amount"""

    currency_id: Required[Annotated[str, PropertyInfo(alias="currencyId")]]
    """The custom currency ID"""

    cost_formula: Annotated[str, PropertyInfo(alias="costFormula")]
    """Optional cost formula expression"""


class PricingModelPricePeriodPrice(TypedDict, total=False):
    """The price amount and currency"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PricingModelPricePeriodTierFlatPrice(TypedDict, total=False):
    """Flat price for this tier"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PricingModelPricePeriodTierUnitPrice(TypedDict, total=False):
    """Per-unit price in this tier"""

    amount: Required[float]
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PricingModelPricePeriodTier(TypedDict, total=False):
    """A tier in tiered pricing."""

    flat_price: Annotated[PricingModelPricePeriodTierFlatPrice, PropertyInfo(alias="flatPrice")]
    """Flat price for this tier"""

    unit_price: Annotated[PricingModelPricePeriodTierUnitPrice, PropertyInfo(alias="unitPrice")]
    """Per-unit price in this tier"""

    up_to: Annotated[float, PropertyInfo(alias="upTo")]
    """Upper bound of this tier (null for unlimited)"""


class PricingModelPricePeriod(TypedDict, total=False):
    """Price configuration for a specific billing period."""

    billing_period: Required[Annotated[Literal["MONTHLY", "ANNUALLY"], PropertyInfo(alias="billingPeriod")]]
    """The billing period (MONTHLY or ANNUALLY)"""

    billing_country_code: Annotated[str, PropertyInfo(alias="billingCountryCode")]
    """ISO country code for localized pricing"""

    block_size: Annotated[float, PropertyInfo(alias="blockSize")]
    """Block size for usage-based pricing"""

    credit_grant_cadence: Annotated[
        Literal["BEGINNING_OF_BILLING_PERIOD", "MONTHLY"], PropertyInfo(alias="creditGrantCadence")
    ]
    """When credits are granted"""

    credit_rate: Annotated[PricingModelPricePeriodCreditRate, PropertyInfo(alias="creditRate")]
    """Credit rate configuration for credit-based pricing"""

    price: PricingModelPricePeriodPrice
    """The price amount and currency"""

    tiers: Iterable[PricingModelPricePeriodTier]
    """Tiered pricing configuration"""


class PricingModelMonthlyResetPeriodConfiguration(TypedDict, total=False):
    """Monthly reset period configuration"""

    according_to: Required[
        Annotated[Literal["SubscriptionStart", "StartOfTheMonth"], PropertyInfo(alias="accordingTo")]
    ]
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class PricingModelWeeklyResetPeriodConfiguration(TypedDict, total=False):
    """Weekly reset period configuration"""

    according_to: Required[
        Annotated[
            Literal[
                "SubscriptionStart",
                "EverySunday",
                "EveryMonday",
                "EveryTuesday",
                "EveryWednesday",
                "EveryThursday",
                "EveryFriday",
                "EverySaturday",
            ],
            PropertyInfo(alias="accordingTo"),
        ]
    ]
    """Reset anchor (SubscriptionStart or specific day)"""


class PricingModelYearlyResetPeriodConfiguration(TypedDict, total=False):
    """Yearly reset period configuration"""

    according_to: Required[Annotated[Literal["SubscriptionStart"], PropertyInfo(alias="accordingTo")]]
    """Reset anchor (SubscriptionStart)"""


class PricingModel(TypedDict, total=False):
    """A pricing model configuration with billing details and price periods."""

    billing_model: Required[
        Annotated[
            Literal["FLAT_FEE", "MINIMUM_SPEND", "PER_UNIT", "USAGE_BASED", "CREDIT_BASED"],
            PropertyInfo(alias="billingModel"),
        ]
    ]
    """The billing model (FLAT_FEE, PER_UNIT, USAGE_BASED, CREDIT_BASED)"""

    price_periods: Required[Annotated[Iterable[PricingModelPricePeriod], PropertyInfo(alias="pricePeriods")]]
    """Array of price period configurations (at least one required)"""

    billing_cadence: Annotated[Literal["RECURRING", "ONE_OFF"], PropertyInfo(alias="billingCadence")]
    """The billing cadence (RECURRING or ONE_OFF)"""

    feature_id: Annotated[str, PropertyInfo(alias="featureId")]
    """The feature ID this pricing model is associated with"""

    max_unit_quantity: Annotated[int, PropertyInfo(alias="maxUnitQuantity")]
    """Maximum number of units (max 999999)"""

    min_unit_quantity: Annotated[int, PropertyInfo(alias="minUnitQuantity")]
    """Minimum number of units"""

    monthly_reset_period_configuration: Annotated[
        PricingModelMonthlyResetPeriodConfiguration, PropertyInfo(alias="monthlyResetPeriodConfiguration")
    ]
    """Monthly reset period configuration"""

    reset_period: Annotated[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"], PropertyInfo(alias="resetPeriod")]
    """The usage reset period"""

    tiers_mode: Annotated[Literal["VOLUME", "GRADUATED"], PropertyInfo(alias="tiersMode")]
    """The tiered pricing mode (VOLUME or GRADUATED)"""

    top_up_custom_currency_id: Annotated[str, PropertyInfo(alias="topUpCustomCurrencyId")]
    """The custom currency ID for top-up pricing"""

    weekly_reset_period_configuration: Annotated[
        PricingModelWeeklyResetPeriodConfiguration, PropertyInfo(alias="weeklyResetPeriodConfiguration")
    ]
    """Weekly reset period configuration"""

    yearly_reset_period_configuration: Annotated[
        PricingModelYearlyResetPeriodConfiguration, PropertyInfo(alias="yearlyResetPeriodConfiguration")
    ]
    """Yearly reset period configuration"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = [
    "PromotionalEntitlementCreateParams",
    "PromotionalEntitlement",
    "PromotionalEntitlementMonthlyResetPeriodConfiguration",
    "PromotionalEntitlementWeeklyResetPeriodConfiguration",
    "PromotionalEntitlementYearlyResetPeriodConfiguration",
]


class PromotionalEntitlementCreateParams(TypedDict, total=False):
    promotional_entitlements: Required[
        Annotated[Iterable[PromotionalEntitlement], PropertyInfo(alias="promotionalEntitlements")]
    ]
    """Promotional entitlements to grant"""


class PromotionalEntitlementMonthlyResetPeriodConfiguration(TypedDict, total=False):
    """
    The monthly reset period configuration of the entitlement, defined when reset period is monthly
    """

    according_to: Required[
        Annotated[Literal["SubscriptionStart", "StartOfTheMonth"], PropertyInfo(alias="accordingTo")]
    ]
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class PromotionalEntitlementWeeklyResetPeriodConfiguration(TypedDict, total=False):
    """
    The weekly reset period configuration of the entitlement, defined when reset period is weekly
    """

    according_to: Required[
        Annotated[
            Literal[
                "SubscriptionStart",
                "EverySunday",
                "EveryMonday",
                "EveryTuesday",
                "EveryWednesday",
                "EveryThursday",
                "EveryFriday",
                "EverySaturday",
            ],
            PropertyInfo(alias="accordingTo"),
        ]
    ]
    """Reset anchor (SubscriptionStart or specific day)"""


class PromotionalEntitlementYearlyResetPeriodConfiguration(TypedDict, total=False):
    """
    The yearly reset period configuration of the entitlement, defined when reset period is yearly
    """

    according_to: Required[Annotated[Literal["SubscriptionStart"], PropertyInfo(alias="accordingTo")]]
    """Reset anchor (SubscriptionStart)"""


class PromotionalEntitlement(TypedDict, total=False):
    """Single entitlement grant config"""

    custom_end_date: Required[
        Annotated[Union[str, datetime, None], PropertyInfo(alias="customEndDate", format="iso8601")]
    ]
    """The custom end date of the promotional entitlement"""

    enum_values: Required[Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="enumValues")]]
    """The enum values of the entitlement"""

    feature_id: Required[Annotated[str, PropertyInfo(alias="featureId")]]
    """The unique identifier of the entitlement feature"""

    has_soft_limit: Required[Annotated[Optional[bool], PropertyInfo(alias="hasSoftLimit")]]
    """Whether the entitlement has a soft limit"""

    has_unlimited_usage: Required[Annotated[Optional[bool], PropertyInfo(alias="hasUnlimitedUsage")]]
    """Whether the entitlement has an unlimited usage"""

    is_visible: Required[Annotated[Optional[bool], PropertyInfo(alias="isVisible")]]
    """Whether the entitlement is visible"""

    monthly_reset_period_configuration: Required[
        Annotated[
            Optional[PromotionalEntitlementMonthlyResetPeriodConfiguration],
            PropertyInfo(alias="monthlyResetPeriodConfiguration"),
        ]
    ]
    """
    The monthly reset period configuration of the entitlement, defined when reset
    period is monthly
    """

    period: Required[Literal["1 week", "1 month", "6 month", "1 year", "lifetime", "custom"]]
    """The grant period of the promotional entitlement"""

    reset_period: Required[
        Annotated[Optional[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"]], PropertyInfo(alias="resetPeriod")]
    ]
    """The reset period of the entitlement"""

    usage_limit: Required[Annotated[Optional[int], PropertyInfo(alias="usageLimit")]]
    """The usage limit of the entitlement"""

    weekly_reset_period_configuration: Required[
        Annotated[
            Optional[PromotionalEntitlementWeeklyResetPeriodConfiguration],
            PropertyInfo(alias="weeklyResetPeriodConfiguration"),
        ]
    ]
    """
    The weekly reset period configuration of the entitlement, defined when reset
    period is weekly
    """

    yearly_reset_period_configuration: Required[
        Annotated[
            Optional[PromotionalEntitlementYearlyResetPeriodConfiguration],
            PropertyInfo(alias="yearlyResetPeriodConfiguration"),
        ]
    ]
    """
    The yearly reset period configuration of the entitlement, defined when reset
    period is yearly
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from stigg.types.v1.events import (
    CreditGetUsageResponse,
    CreditListLedgerResponse,
    CreditGetAutoRechargeResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCredits:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_auto_recharge(self, client: Stigg) -> None:
        credit = client.v1.events.credits.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        )
        assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_auto_recharge(self, client: Stigg) -> None:
        response = client.v1.events.credits.with_raw_response.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = response.parse()
        assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_auto_recharge(self, client: Stigg) -> None:
        with client.v1.events.credits.with_streaming_response.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = response.parse()
            assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_usage(self, client: Stigg) -> None:
        credit = client.v1.events.credits.get_usage(
            customer_id="customerId",
        )
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_usage_with_all_params(self, client: Stigg) -> None:
        credit = client.v1.events.credits.get_usage(
            customer_id="customerId",
            currency_id="currencyId",
            resource_id="resourceId",
            time_range="LAST_DAY",
        )
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_usage(self, client: Stigg) -> None:
        response = client.v1.events.credits.with_raw_response.get_usage(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = response.parse()
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_usage(self, client: Stigg) -> None:
        with client.v1.events.credits.with_streaming_response.get_usage(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = response.parse()
            assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_ledger(self, client: Stigg) -> None:
        credit = client.v1.events.credits.list_ledger(
            customer_id="customerId",
        )
        assert_matches_type(SyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_ledger_with_all_params(self, client: Stigg) -> None:
        credit = client.v1.events.credits.list_ledger(
            customer_id="customerId",
            after="after",
            before="before",
            currency_id="currencyId",
            limit=1,
            resource_id="resourceId",
        )
        assert_matches_type(SyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_ledger(self, client: Stigg) -> None:
        response = client.v1.events.credits.with_raw_response.list_ledger(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = response.parse()
        assert_matches_type(SyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_ledger(self, client: Stigg) -> None:
        with client.v1.events.credits.with_streaming_response.list_ledger(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = response.parse()
            assert_matches_type(SyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCredits:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_auto_recharge(self, async_client: AsyncStigg) -> None:
        credit = await async_client.v1.events.credits.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        )
        assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_auto_recharge(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.with_raw_response.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = await response.parse()
        assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_auto_recharge(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.with_streaming_response.get_auto_recharge(
            currency_id="currencyId",
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = await response.parse()
            assert_matches_type(CreditGetAutoRechargeResponse, credit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_usage(self, async_client: AsyncStigg) -> None:
        credit = await async_client.v1.events.credits.get_usage(
            customer_id="customerId",
        )
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_usage_with_all_params(self, async_client: AsyncStigg) -> None:
        credit = await async_client.v1.events.credits.get_usage(
            customer_id="customerId",
            currency_id="currencyId",
            resource_id="resourceId",
            time_range="LAST_DAY",
        )
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_usage(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.with_raw_response.get_usage(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = await response.parse()
        assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_usage(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.with_streaming_response.get_usage(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = await response.parse()
            assert_matches_type(CreditGetUsageResponse, credit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_ledger(self, async_client: AsyncStigg) -> None:
        credit = await async_client.v1.events.credits.list_ledger(
            customer_id="customerId",
        )
        assert_matches_type(AsyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_ledger_with_all_params(self, async_client: AsyncStigg) -> None:
        credit = await async_client.v1.events.credits.list_ledger(
            customer_id="customerId",
            after="after",
            before="before",
            currency_id="currencyId",
            limit=1,
            resource_id="resourceId",
        )
        assert_matches_type(AsyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_ledger(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.with_raw_response.list_ledger(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_ledger(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.with_streaming_response.list_ledger(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[CreditListLedgerResponse], credit, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .grants import (
    GrantsResource,
    AsyncGrantsResource,
    GrantsResourceWithRawResponse,
    AsyncGrantsResourceWithRawResponse,
    GrantsResourceWithStreamingResponse,
    AsyncGrantsResourceWithStreamingResponse,
)
from .credits import (
    CreditsResource,
    AsyncCreditsResource,
    CreditsResourceWithRawResponse,
    AsyncCreditsResourceWithRawResponse,
    CreditsResourceWithStreamingResponse,
    AsyncCreditsResourceWithStreamingResponse,
)

__all__ = [
    "GrantsResource",
    "AsyncGrantsResource",
    "GrantsResourceWithRawResponse",
    "AsyncGrantsResourceWithRawResponse",
    "GrantsResourceWithStreamingResponse",
    "AsyncGrantsResourceWithStreamingResponse",
    "CreditsResource",
    "AsyncCreditsResource",
    "CreditsResourceWithRawResponse",
    "AsyncCreditsResourceWithRawResponse",
    "CreditsResourceWithStreamingResponse",
    "AsyncCreditsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg.types.v1 import CustomerResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPaymentMethod:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_attach(self, client: Stigg) -> None:
        payment_method = client.v1.customers.payment_method.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_attach_with_all_params(self, client: Stigg) -> None:
        payment_method = client.v1.customers.payment_method.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
            billing_currency="usd",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_attach(self, client: Stigg) -> None:
        response = client.v1.customers.payment_method.with_raw_response.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_method = response.parse()
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_attach(self, client: Stigg) -> None:
        with client.v1.customers.payment_method.with_streaming_response.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_method = response.parse()
            assert_matches_type(CustomerResponse, payment_method, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_attach(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.payment_method.with_raw_response.attach(
                id="",
                integration_id="integrationId",
                payment_method_id="paymentMethodId",
                vendor_identifier="AUTH0",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_detach(self, client: Stigg) -> None:
        payment_method = client.v1.customers.payment_method.detach(
            "x",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_detach(self, client: Stigg) -> None:
        response = client.v1.customers.payment_method.with_raw_response.detach(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_method = response.parse()
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_detach(self, client: Stigg) -> None:
        with client.v1.customers.payment_method.with_streaming_response.detach(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_method = response.parse()
            assert_matches_type(CustomerResponse, payment_method, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_detach(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.payment_method.with_raw_response.detach(
                "",
            )


class TestAsyncPaymentMethod:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_attach(self, async_client: AsyncStigg) -> None:
        payment_method = await async_client.v1.customers.payment_method.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_attach_with_all_params(self, async_client: AsyncStigg) -> None:
        payment_method = await async_client.v1.customers.payment_method.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
            billing_currency="usd",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_attach(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.payment_method.with_raw_response.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_method = await response.parse()
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_attach(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.payment_method.with_streaming_response.attach(
            id="x",
            integration_id="integrationId",
            payment_method_id="paymentMethodId",
            vendor_identifier="AUTH0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_method = await response.parse()
            assert_matches_type(CustomerResponse, payment_method, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_attach(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.payment_method.with_raw_response.attach(
                id="",
                integration_id="integrationId",
                payment_method_id="paymentMethodId",
                vendor_identifier="AUTH0",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_detach(self, async_client: AsyncStigg) -> None:
        payment_method = await async_client.v1.customers.payment_method.detach(
            "x",
        )
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_detach(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.payment_method.with_raw_response.detach(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_method = await response.parse()
        assert_matches_type(CustomerResponse, payment_method, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_detach(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.payment_method.with_streaming_response.detach(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_method = await response.parse()
            assert_matches_type(CustomerResponse, payment_method, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_detach(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.payment_method.with_raw_response.detach(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from stigg.types.v1.events.credits import GrantListResponse, CreditGrantResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGrants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Stigg) -> None:
        grant = client.v1.events.credits.grants.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Stigg) -> None:
        grant = client.v1.events.credits.grants.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
            await_payment_confirmation=True,
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "invoice_days_until_due": 0,
                "is_invoice_paid": True,
            },
            comment="comment",
            cost={
                "amount": 0,
                "currency": "usd",
            },
            effective_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            expire_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            metadata={"foo": "string"},
            payment_collection_method="CHARGE",
            priority=0,
            resource_id="resourceId",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Stigg) -> None:
        response = client.v1.events.credits.grants.with_raw_response.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = response.parse()
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Stigg) -> None:
        with client.v1.events.credits.grants.with_streaming_response.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = response.parse()
            assert_matches_type(CreditGrantResponse, grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        grant = client.v1.events.credits.grants.list(
            customer_id="customerId",
        )
        assert_matches_type(SyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Stigg) -> None:
        grant = client.v1.events.credits.grants.list(
            customer_id="customerId",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            currency_id="currencyId",
            limit=1,
            resource_id="resourceId",
        )
        assert_matches_type(SyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.events.credits.grants.with_raw_response.list(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = response.parse()
        assert_matches_type(SyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.events.credits.grants.with_streaming_response.list(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = response.parse()
            assert_matches_type(SyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_void(self, client: Stigg) -> None:
        grant = client.v1.events.credits.grants.void(
            "x",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_void(self, client: Stigg) -> None:
        response = client.v1.events.credits.grants.with_raw_response.void(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = response.parse()
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_void(self, client: Stigg) -> None:
        with client.v1.events.credits.grants.with_streaming_response.void(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = response.parse()
            assert_matches_type(CreditGrantResponse, grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_void(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.events.credits.grants.with_raw_response.void(
                "",
            )


class TestAsyncGrants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncStigg) -> None:
        grant = await async_client.v1.events.credits.grants.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncStigg) -> None:
        grant = await async_client.v1.events.credits.grants.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
            await_payment_confirmation=True,
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "invoice_days_until_due": 0,
                "is_invoice_paid": True,
            },
            comment="comment",
            cost={
                "amount": 0,
                "currency": "usd",
            },
            effective_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            expire_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            metadata={"foo": "string"},
            payment_collection_method="CHARGE",
            priority=0,
            resource_id="resourceId",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.grants.with_raw_response.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = await response.parse()
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.grants.with_streaming_response.create(
            amount=0,
            currency_id="currencyId",
            customer_id="customerId",
            display_name="displayName",
            grant_type="PAID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = await response.parse()
            assert_matches_type(CreditGrantResponse, grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        grant = await async_client.v1.events.credits.grants.list(
            customer_id="customerId",
        )
        assert_matches_type(AsyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncStigg) -> None:
        grant = await async_client.v1.events.credits.grants.list(
            customer_id="customerId",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            currency_id="currencyId",
            limit=1,
            resource_id="resourceId",
        )
        assert_matches_type(AsyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.grants.with_raw_response.list(
            customer_id="customerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.grants.with_streaming_response.list(
            customer_id="customerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[GrantListResponse], grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_void(self, async_client: AsyncStigg) -> None:
        grant = await async_client.v1.events.credits.grants.void(
            "x",
        )
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_void(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.events.credits.grants.with_raw_response.void(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        grant = await response.parse()
        assert_matches_type(CreditGrantResponse, grant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_void(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.events.credits.grants.with_streaming_response.void(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            grant = await response.parse()
            assert_matches_type(CreditGrantResponse, grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_void(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.events.credits.grants.with_raw_response.void(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ProductCreateProductParams"]


class ProductCreateProductParams(TypedDict, total=False):
    id: Required[str]
    """The unique identifier for the entity"""

    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """Display name of the product"""

    description: Optional[str]
    """Description of the product"""

    metadata: Optional[Dict[str, str]]
    """Additional metadata for the product"""

    multiple_subscriptions: Annotated[bool, PropertyInfo(alias="multipleSubscriptions")]
    """Indicates if multiple subscriptions to this product are allowed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import (
    Product,
    ProductListProductsResponse,
)
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProducts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive_product(self, client: Stigg) -> None:
        product = client.v1.products.archive_product(
            "x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_archive_product(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.archive_product(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_archive_product(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.archive_product(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_archive_product(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.products.with_raw_response.archive_product(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_product(self, client: Stigg) -> None:
        product = client.v1.products.create_product(
            id="id",
            display_name="displayName",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_product_with_all_params(self, client: Stigg) -> None:
        product = client.v1.products.create_product(
            id="id",
            display_name="displayName",
            description="description",
            metadata={"foo": "string"},
            multiple_subscriptions=True,
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_product(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.create_product(
            id="id",
            display_name="displayName",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_product(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.create_product(
            id="id",
            display_name="displayName",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_duplicate_product(self, client: Stigg) -> None:
        product = client.v1.products.duplicate_product(
            path_id="x",
            body_id="id",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_duplicate_product_with_all_params(self, client: Stigg) -> None:
        product = client.v1.products.duplicate_product(
            path_id="x",
            body_id="id",
            description="description",
            display_name="displayName",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_duplicate_product(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.duplicate_product(
            path_id="x",
            body_id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_duplicate_product(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.duplicate_product(
            path_id="x",
            body_id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_duplicate_product(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            client.v1.products.with_raw_response.duplicate_product(
                path_id="",
                body_id="id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_products(self, client: Stigg) -> None:
        product = client.v1.products.list_products()
        assert_matches_type(SyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_products_with_all_params(self, client: Stigg) -> None:
        product = client.v1.products.list_products(
            id="id",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            status="status",
        )
        assert_matches_type(SyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_products(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.list_products()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(SyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_products(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.list_products() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(SyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_unarchive_product(self, client: Stigg) -> None:
        product = client.v1.products.unarchive_product(
            "x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_unarchive_product(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.unarchive_product(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_unarchive_product(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.unarchive_product(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_unarchive_product(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.products.with_raw_response.unarchive_product(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_product(self, client: Stigg) -> None:
        product = client.v1.products.update_product(
            id="x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_product_with_all_params(self, client: Stigg) -> None:
        product = client.v1.products.update_product(
            id="x",
            description="description",
            display_name="displayName",
            metadata={"foo": "string"},
            multiple_subscriptions=True,
            product_settings={
                "subscription_cancellation_time": "END_OF_BILLING_PERIOD",
                "subscription_end_setup": "DOWNGRADE_TO_FREE",
                "subscription_start_setup": "PLAN_SELECTION",
                "downgrade_plan_id": "downgradePlanId",
                "prorate_at_end_of_billing_period": True,
                "subscription_start_plan_id": "subscriptionStartPlanId",
            },
            usage_reset_cutoff_rule={"behavior": "NEVER_RESET"},
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_product(self, client: Stigg) -> None:
        response = client.v1.products.with_raw_response.update_product(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_product(self, client: Stigg) -> None:
        with client.v1.products.with_streaming_response.update_product(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_product(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.products.with_raw_response.update_product(
                id="",
            )


class TestAsyncProducts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive_product(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.archive_product(
            "x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_archive_product(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.archive_product(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_archive_product(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.archive_product(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_archive_product(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.products.with_raw_response.archive_product(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_product(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.create_product(
            id="id",
            display_name="displayName",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_product_with_all_params(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.create_product(
            id="id",
            display_name="displayName",
            description="description",
            metadata={"foo": "string"},
            multiple_subscriptions=True,
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_product(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.create_product(
            id="id",
            display_name="displayName",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_product(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.create_product(
            id="id",
            display_name="displayName",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_duplicate_product(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.duplicate_product(
            path_id="x",
            body_id="id",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_duplicate_product_with_all_params(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.duplicate_product(
            path_id="x",
            body_id="id",
            description="description",
            display_name="displayName",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_duplicate_product(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.duplicate_product(
            path_id="x",
            body_id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_duplicate_product(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.duplicate_product(
            path_id="x",
            body_id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_duplicate_product(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            await async_client.v1.products.with_raw_response.duplicate_product(
                path_id="",
                body_id="id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_products(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.list_products()
        assert_matches_type(AsyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_products_with_all_params(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.list_products(
            id="id",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            status="status",
        )
        assert_matches_type(AsyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_products(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.list_products()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_products(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.list_products() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[ProductListProductsResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_unarchive_product(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.unarchive_product(
            "x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_unarchive_product(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.unarchive_product(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_unarchive_product(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.unarchive_product(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_unarchive_product(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.products.with_raw_response.unarchive_product(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_product(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.update_product(
            id="x",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_product_with_all_params(self, async_client: AsyncStigg) -> None:
        product = await async_client.v1.products.update_product(
            id="x",
            description="description",
            display_name="displayName",
            metadata={"foo": "string"},
            multiple_subscriptions=True,
            product_settings={
                "subscription_cancellation_time": "END_OF_BILLING_PERIOD",
                "subscription_end_setup": "DOWNGRADE_TO_FREE",
                "subscription_start_setup": "PLAN_SELECTION",
                "downgrade_plan_id": "downgradePlanId",
                "prorate_at_end_of_billing_period": True,
                "subscription_start_plan_id": "subscriptionStartPlanId",
            },
            usage_reset_cutoff_rule={"behavior": "NEVER_RESET"},
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_product(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.products.with_raw_response.update_product(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_product(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.products.with_streaming_response.update_product(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_product(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.products.with_raw_response.update_product(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CouponListResponse", "AmountsOff"]


class AmountsOff(BaseModel):
    """Monetary amount with currency"""

    amount: float
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """ISO 4217 currency code"""


class CouponListResponse(BaseModel):
    """Discount instrument with percentage or fixed amount"""

    id: str
    """The unique identifier for the entity"""

    amounts_off: Optional[List[AmountsOff]] = FieldInfo(alias="amountsOff", default=None)
    """Fixed amount discounts in different currencies"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The unique identifier for the entity in the billing provider"""

    billing_link_url: Optional[str] = FieldInfo(alias="billingLinkUrl", default=None)
    """The URL to the entity in the billing provider"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """Description of the coupon"""

    duration_in_months: Optional[int] = FieldInfo(alias="durationInMonths", default=None)
    """Duration of the coupon validity in months"""

    metadata: Optional[Dict[str, str]] = None
    """Metadata associated with the entity"""

    name: str
    """Name of the coupon"""

    percent_off: Optional[int] = FieldInfo(alias="percentOff", default=None)
    """Percentage discount off the original price"""

    source: Optional[Literal["STIGG", "STIGG_ADHOC", "STRIPE"]] = None
    """The source of the coupon"""

    status: Literal["ACTIVE", "ARCHIVED"]
    """Current status of the coupon"""

    type: Literal["FIXED", "PERCENTAGE"]
    """Type of the coupon (percentage or fixed amount)"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CreditGetAutoRechargeParams"]


class CreditGetAutoRechargeParams(TypedDict, total=False):
    currency_id: Required[Annotated[str, PropertyInfo(alias="currencyId")]]
    """Filter by currency ID (required)"""

    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Filter by customer ID (required)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from ._models import BaseModel
from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["MyCursorIDPagePagination", "SyncMyCursorIDPage", "AsyncMyCursorIDPage"]

_T = TypeVar("_T")


class MyCursorIDPagePagination(BaseModel):
    next: Optional[str] = None

    prev: Optional[str] = None


class SyncMyCursorIDPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    pagination: Optional[MyCursorIDPagePagination] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        if self._options.params.get("before"):
            prev = None
            if self.pagination is not None:
                if self.pagination.prev is not None:
                    prev = self.pagination.prev
            if not prev:
                return None

            return PageInfo(params={"before": prev})

        next = None
        if self.pagination is not None:
            if self.pagination.next is not None:
                next = self.pagination.next
        if not next:
            return None

        return PageInfo(params={"after": next})


class AsyncMyCursorIDPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    pagination: Optional[MyCursorIDPagePagination] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        if self._options.params.get("before"):
            prev = None
            if self.pagination is not None:
                if self.pagination.prev is not None:
                    prev = self.pagination.prev
            if not prev:
                return None

            return PageInfo(params={"before": prev})

        next = None
        if self.pagination is not None:
            if self.pagination.next is not None:
                next = self.pagination.next
        if not next:
            return None

        return PageInfo(params={"after": next})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.customers import promotional_entitlement_list_params, promotional_entitlement_create_params
from ....types.v1.customers.promotional_entitlement_list_response import PromotionalEntitlementListResponse
from ....types.v1.customers.promotional_entitlement_create_response import PromotionalEntitlementCreateResponse
from ....types.v1.customers.promotional_entitlement_revoke_response import PromotionalEntitlementRevokeResponse

__all__ = ["PromotionalEntitlementsResource", "AsyncPromotionalEntitlementsResource"]


class PromotionalEntitlementsResource(SyncAPIResource):
    """Operations related to promotional entitlements"""

    @cached_property
    def with_raw_response(self) -> PromotionalEntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return PromotionalEntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PromotionalEntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return PromotionalEntitlementsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        promotional_entitlements: Iterable[promotional_entitlement_create_params.PromotionalEntitlement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PromotionalEntitlementCreateResponse:
        """
        Grants promotional entitlements to a customer, providing feature access outside
        their subscription. Entitlements can be time-limited or permanent.

        Args:
          promotional_entitlements: Promotional entitlements to grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/customers/{id}/promotional-entitlements",
            body=maybe_transform(
                {"promotional_entitlements": promotional_entitlements},
                promotional_entitlement_create_params.PromotionalEntitlementCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromotionalEntitlementCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: promotional_entitlement_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[PromotionalEntitlementListResponse]:
        """
        Retrieves a paginated list of a customer's promotional entitlements.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by promotional entitlement status. Supports comma-separated values for
              multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/api/v1/customers/{id}/promotional-entitlements",
            page=SyncMyCursorIDPage[PromotionalEntitlementListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                    },
                    promotional_entitlement_list_params.PromotionalEntitlementListParams,
                ),
            ),
            model=PromotionalEntitlementListResponse,
        )

    def revoke(
        self,
        feature_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PromotionalEntitlementRevokeResponse:
        """
        Revokes a previously granted promotional entitlement from a customer for a
        specific feature.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not feature_id:
            raise ValueError(f"Expected a non-empty value for `feature_id` but received {feature_id!r}")
        return self._delete(
            f"/api/v1/customers/{id}/promotional-entitlements/{feature_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromotionalEntitlementRevokeResponse,
        )


class AsyncPromotionalEntitlementsResource(AsyncAPIResource):
    """Operations related to promotional entitlements"""

    @cached_property
    def with_raw_response(self) -> AsyncPromotionalEntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPromotionalEntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPromotionalEntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncPromotionalEntitlementsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        promotional_entitlements: Iterable[promotional_entitlement_create_params.PromotionalEntitlement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PromotionalEntitlementCreateResponse:
        """
        Grants promotional entitlements to a customer, providing feature access outside
        their subscription. Entitlements can be time-limited or permanent.

        Args:
          promotional_entitlements: Promotional entitlements to grant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/customers/{id}/promotional-entitlements",
            body=await async_maybe_transform(
                {"promotional_entitlements": promotional_entitlements},
                promotional_entitlement_create_params.PromotionalEntitlementCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromotionalEntitlementCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: promotional_entitlement_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PromotionalEntitlementListResponse, AsyncMyCursorIDPage[PromotionalEntitlementListResponse]]:
        """
        Retrieves a paginated list of a customer's promotional entitlements.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by promotional entitlement status. Supports comma-separated values for
              multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/api/v1/customers/{id}/promotional-entitlements",
            page=AsyncMyCursorIDPage[PromotionalEntitlementListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                    },
                    promotional_entitlement_list_params.PromotionalEntitlementListParams,
                ),
            ),
            model=PromotionalEntitlementListResponse,
        )

    async def revoke(
        self,
        feature_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PromotionalEntitlementRevokeResponse:
        """
        Revokes a previously granted promotional entitlement from a customer for a
        specific feature.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not feature_id:
            raise ValueError(f"Expected a non-empty value for `feature_id` but received {feature_id!r}")
        return await self._delete(
            f"/api/v1/customers/{id}/promotional-entitlements/{feature_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromotionalEntitlementRevokeResponse,
        )


class PromotionalEntitlementsResourceWithRawResponse:
    def __init__(self, promotional_entitlements: PromotionalEntitlementsResource) -> None:
        self._promotional_entitlements = promotional_entitlements

        self.create = to_raw_response_wrapper(
            promotional_entitlements.create,
        )
        self.list = to_raw_response_wrapper(
            promotional_entitlements.list,
        )
        self.revoke = to_raw_response_wrapper(
            promotional_entitlements.revoke,
        )


class AsyncPromotionalEntitlementsResourceWithRawResponse:
    def __init__(self, promotional_entitlements: AsyncPromotionalEntitlementsResource) -> None:
        self._promotional_entitlements = promotional_entitlements

        self.create = async_to_raw_response_wrapper(
            promotional_entitlements.create,
        )
        self.list = async_to_raw_response_wrapper(
            promotional_entitlements.list,
        )
        self.revoke = async_to_raw_response_wrapper(
            promotional_entitlements.revoke,
        )


class PromotionalEntitlementsResourceWithStreamingResponse:
    def __init__(self, promotional_entitlements: PromotionalEntitlementsResource) -> None:
        self._promotional_entitlements = promotional_entitlements

        self.create = to_streamed_response_wrapper(
            promotional_entitlements.create,
        )
        self.list = to_streamed_response_wrapper(
            promotional_entitlements.list,
        )
        self.revoke = to_streamed_response_wrapper(
            promotional_entitlements.revoke,
        )


class AsyncPromotionalEntitlementsResourceWithStreamingResponse:
    def __init__(self, promotional_entitlements: AsyncPromotionalEntitlementsResource) -> None:
        self._promotional_entitlements = promotional_entitlements

        self.create = async_to_streamed_response_wrapper(
            promotional_entitlements.create,
        )
        self.list = async_to_streamed_response_wrapper(
            promotional_entitlements.list,
        )
        self.revoke = async_to_streamed_response_wrapper(
            promotional_entitlements.revoke,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["GrantCreateParams", "BillingInformation", "BillingInformationBillingAddress", "Cost"]


class GrantCreateParams(TypedDict, total=False):
    amount: Required[float]
    """The credit amount to grant"""

    currency_id: Required[Annotated[str, PropertyInfo(alias="currencyId")]]
    """The credit currency ID (required)"""

    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """The customer ID to grant credits to (required)"""

    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """The display name for the credit grant"""

    grant_type: Required[Annotated[Literal["PAID", "PROMOTIONAL", "RECURRING"], PropertyInfo(alias="grantType")]]
    """The type of credit grant (PAID, PROMOTIONAL, RECURRING)"""

    await_payment_confirmation: Annotated[bool, PropertyInfo(alias="awaitPaymentConfirmation")]
    """Whether to wait for payment confirmation before returning (default: true)"""

    billing_information: Annotated[BillingInformation, PropertyInfo(alias="billingInformation")]
    """Billing information for the credit grant"""

    comment: str
    """An optional comment on the credit grant"""

    cost: Cost
    """The monetary cost of the credit grant"""

    effective_at: Annotated[Union[str, datetime], PropertyInfo(alias="effectiveAt", format="iso8601")]
    """The date when the credit grant becomes effective"""

    expire_at: Annotated[Union[str, datetime], PropertyInfo(alias="expireAt", format="iso8601")]
    """The date when the credit grant expires"""

    metadata: Dict[str, str]
    """Additional metadata for the credit grant"""

    payment_collection_method: Annotated[
        Literal["CHARGE", "INVOICE", "NONE"], PropertyInfo(alias="paymentCollectionMethod")
    ]
    """The payment collection method (CHARGE, INVOICE, NONE)"""

    priority: int
    """The priority of the credit grant (lower number = higher priority)"""

    resource_id: Annotated[str, PropertyInfo(alias="resourceId")]
    """The resource ID to scope the grant to"""


class BillingInformationBillingAddress(TypedDict, total=False):
    """The billing address"""

    city: str
    """City name"""

    country: str
    """Country code or name"""

    line1: str
    """Street address line 1"""

    line2: str
    """Street address line 2"""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Postal or ZIP code"""

    state: str
    """State or province"""


class BillingInformation(TypedDict, total=False):
    """Billing information for the credit grant"""

    billing_address: Annotated[BillingInformationBillingAddress, PropertyInfo(alias="billingAddress")]
    """The billing address"""

    invoice_days_until_due: Annotated[float, PropertyInfo(alias="invoiceDaysUntilDue")]
    """Days until the invoice is due"""

    is_invoice_paid: Annotated[bool, PropertyInfo(alias="isInvoicePaid")]
    """Whether the invoice is already paid"""


class Cost(TypedDict, total=False):
    """The monetary cost of the credit grant"""

    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """ISO 4217 currency code"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["SubscriptionImportResponse", "Data"]


class Data(BaseModel):
    task_id: str = FieldInfo(alias="taskId")
    """Unique identifier for the import task"""


class SubscriptionImportResponse(BaseModel):
    """Response object"""

    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CreditListLedgerResponse"]


class CreditListLedgerResponse(BaseModel):
    """A credit ledger event representing a change to credit balance"""

    amount: float
    """The credit amount for this event"""

    credit_currency_id: str = FieldInfo(alias="creditCurrencyId")
    """The credit currency ID"""

    credit_grant_id: str = FieldInfo(alias="creditGrantId")
    """The credit grant ID associated with this event"""

    customer_id: str = FieldInfo(alias="customerId")
    """The customer ID this event belongs to"""

    event_id: Optional[str] = FieldInfo(alias="eventId", default=None)
    """The unique event identifier"""

    event_type: Literal[
        "CREDITS_GRANTED", "CREDITS_EXPIRED", "CREDITS_CONSUMED", "CREDITS_VOIDED", "CREDITS_UPDATED"
    ] = FieldInfo(alias="eventType")
    """The type of credit event"""

    feature_id: Optional[str] = FieldInfo(alias="featureId", default=None)
    """The feature ID associated with this event"""

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """The resource ID this event is scoped to"""

    timestamp: datetime
    """The timestamp when the event occurred"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .grant_list_params import GrantListParams as GrantListParams
from .grant_create_params import GrantCreateParams as GrantCreateParams
from .grant_list_response import GrantListResponse as GrantListResponse
from .credit_grant_response import CreditGrantResponse as CreditGrantResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "CustomerListResponse",
    "DefaultPaymentMethod",
    "Integration",
    "Passthrough",
    "PassthroughStripe",
    "PassthroughStripeBillingAddress",
    "PassthroughStripeShippingAddress",
    "PassthroughStripeTaxID",
    "PassthroughZuora",
    "PassthroughZuoraBillingAddress",
]


class DefaultPaymentMethod(BaseModel):
    """The default payment method details"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The default payment method id"""

    card_expiry_month: Optional[float] = FieldInfo(alias="cardExpiryMonth", default=None)
    """The expiration month of the default payment method"""

    card_expiry_year: Optional[float] = FieldInfo(alias="cardExpiryYear", default=None)
    """The expiration year of the default payment method"""

    card_last4_digits: Optional[str] = FieldInfo(alias="cardLast4Digits", default=None)
    """The last 4 digits of the default payment method"""

    type: Literal["CARD", "BANK", "CASH_APP"]
    """The default payment method type"""


class Integration(BaseModel):
    """External billing or CRM integration link"""

    id: str
    """Integration details"""

    synced_entity_id: Optional[str] = FieldInfo(alias="syncedEntityId", default=None)
    """Synced entity id"""

    vendor_identifier: Literal[
        "AUTH0",
        "ZUORA",
        "STRIPE",
        "HUBSPOT",
        "AWS_MARKETPLACE",
        "SNOWFLAKE",
        "SALESFORCE",
        "BIG_QUERY",
        "OPEN_FGA",
        "APP_STORE",
    ] = FieldInfo(alias="vendorIdentifier")
    """The vendor identifier of integration"""


class PassthroughStripeBillingAddress(BaseModel):
    """Physical address"""

    city: Optional[str] = None
    """City name"""

    country: Optional[str] = None
    """Country code or name"""

    line1: Optional[str] = None
    """Street address line 1"""

    line2: Optional[str] = None
    """Street address line 2"""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """Postal or ZIP code"""

    state: Optional[str] = None
    """State or province"""


class PassthroughStripeShippingAddress(BaseModel):
    """Physical address"""

    city: Optional[str] = None
    """City name"""

    country: Optional[str] = None
    """Country code or name"""

    line1: Optional[str] = None
    """Street address line 1"""

    line2: Optional[str] = None
    """Street address line 2"""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """Postal or ZIP code"""

    state: Optional[str] = None
    """State or province"""


class PassthroughStripeTaxID(BaseModel):
    """Tax identifier with type and value for customer tax exemptions."""

    type: str
    """The type of tax exemption identifier, such as VAT."""

    value: str
    """The actual tax identifier value"""


class PassthroughStripe(BaseModel):
    """Stripe-specific billing fields for the customer."""

    billing_address: Optional[PassthroughStripeBillingAddress] = FieldInfo(alias="billingAddress", default=None)
    """Physical address"""

    customer_name: Optional[str] = FieldInfo(alias="customerName", default=None)
    """Customer name"""

    invoice_custom_fields: Optional[Dict[str, str]] = FieldInfo(alias="invoiceCustomFields", default=None)
    """Invoice custom fields"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata"""

    payment_method_id: Optional[str] = FieldInfo(alias="paymentMethodId", default=None)
    """Billing provider payment method id, attached to this customer"""

    shipping_address: Optional[PassthroughStripeShippingAddress] = FieldInfo(alias="shippingAddress", default=None)
    """Physical address"""

    tax_ids: Optional[List[PassthroughStripeTaxID]] = FieldInfo(alias="taxIds", default=None)
    """Tax IDs"""


class PassthroughZuoraBillingAddress(BaseModel):
    """Physical address"""

    city: Optional[str] = None
    """City name"""

    country: Optional[str] = None
    """Country code or name"""

    line1: Optional[str] = None
    """Street address line 1"""

    line2: Optional[str] = None
    """Street address line 2"""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """Postal or ZIP code"""

    state: Optional[str] = None
    """State or province"""


class PassthroughZuora(BaseModel):
    """Zuora-specific billing fields for the customer."""

    billing_address: Optional[PassthroughZuoraBillingAddress] = FieldInfo(alias="billingAddress", default=None)
    """Physical address"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """Customers selected currency"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata"""

    payment_method_id: Optional[str] = FieldInfo(alias="paymentMethodId", default=None)
    """Billing provider payment method id, attached to this customer"""


class Passthrough(BaseModel):
    """Vendor-specific billing passthrough fields."""

    stripe: Optional[PassthroughStripe] = None
    """Stripe-specific billing fields for the customer."""

    zuora: Optional[PassthroughZuora] = None
    """Zuora-specific billing fields for the customer."""


class CustomerListResponse(BaseModel):
    """A customer can be either an organization or an individual"""

    id: str
    """Customer slug"""

    archived_at: Optional[datetime] = FieldInfo(alias="archivedAt", default=None)
    """Timestamp of when the record was deleted"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    billing_currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = FieldInfo(alias="billingCurrency", default=None)
    """The billing currency of the customer"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The unique identifier for the entity in the billing provider"""

    coupon_id: Optional[str] = FieldInfo(alias="couponId", default=None)
    """Customer level coupon"""

    default_payment_method: Optional[DefaultPaymentMethod] = FieldInfo(alias="defaultPaymentMethod", default=None)
    """The default payment method details"""

    email: Optional[str] = None
    """The email of the customer"""

    integrations: Optional[List[Integration]] = None
    """List of integrations"""

    language: Optional[str] = None
    """Language to use for this customer"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata"""

    name: Optional[str] = None
    """The name of the customer"""

    passthrough: Optional[Passthrough] = None
    """Vendor-specific billing passthrough fields."""

    timezone: Optional[str] = None
    """Timezone to use for this customer"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "CustomerProvisionParams",
    "DefaultPaymentMethod",
    "Integration",
    "Passthrough",
    "PassthroughStripe",
    "PassthroughStripeBillingAddress",
    "PassthroughStripeShippingAddress",
    "PassthroughStripeTaxID",
    "PassthroughZuora",
    "PassthroughZuoraBillingAddress",
]


class CustomerProvisionParams(TypedDict, total=False):
    id: Required[str]
    """Customer slug"""

    billing_currency: Annotated[
        Optional[
            Literal[
                "usd",
                "aed",
                "all",
                "amd",
                "ang",
                "aud",
                "awg",
                "azn",
                "bam",
                "bbd",
                "bdt",
                "bgn",
                "bif",
                "bmd",
                "bnd",
                "bsd",
                "bwp",
                "byn",
                "bzd",
                "brl",
                "cad",
                "cdf",
                "chf",
                "cny",
                "czk",
                "dkk",
                "dop",
                "dzd",
                "egp",
                "etb",
                "eur",
                "fjd",
                "gbp",
                "gel",
                "gip",
                "gmd",
                "gyd",
                "hkd",
                "hrk",
                "htg",
                "idr",
                "ils",
                "inr",
                "isk",
                "jmd",
                "jpy",
                "kes",
                "kgs",
                "khr",
                "kmf",
                "krw",
                "kyd",
                "kzt",
                "lbp",
                "lkr",
                "lrd",
                "lsl",
                "mad",
                "mdl",
                "mga",
                "mkd",
                "mmk",
                "mnt",
                "mop",
                "mro",
                "mvr",
                "mwk",
                "mxn",
                "myr",
                "mzn",
                "nad",
                "ngn",
                "nok",
                "npr",
                "nzd",
                "pgk",
                "php",
                "pkr",
                "pln",
                "qar",
                "ron",
                "rsd",
                "rub",
                "rwf",
                "sar",
                "sbd",
                "scr",
                "sek",
                "sgd",
                "sle",
                "sll",
                "sos",
                "szl",
                "thb",
                "tjs",
                "top",
                "try",
                "ttd",
                "tzs",
                "uah",
                "uzs",
                "vnd",
                "vuv",
                "wst",
                "xaf",
                "xcd",
                "yer",
                "zar",
                "zmw",
                "clp",
                "djf",
                "gnf",
                "ugx",
                "pyg",
                "xof",
                "xpf",
            ]
        ],
        PropertyInfo(alias="billingCurrency"),
    ]
    """The billing currency of the customer"""

    billing_id: Annotated[Optional[str], PropertyInfo(alias="billingId")]
    """The unique identifier for the entity in the billing provider"""

    coupon_id: Annotated[Optional[str], PropertyInfo(alias="couponId")]
    """Customer level coupon"""

    default_payment_method: Annotated[Optional[DefaultPaymentMethod], PropertyInfo(alias="defaultPaymentMethod")]
    """The default payment method details"""

    email: Optional[str]
    """The email of the customer"""

    integrations: Iterable[Integration]
    """List of integrations"""

    language: Optional[str]
    """Language to use for this customer"""

    metadata: Dict[str, str]
    """Additional metadata"""

    name: Optional[str]
    """The name of the customer"""

    passthrough: Passthrough
    """Vendor-specific billing passthrough fields."""

    timezone: Optional[str]
    """Timezone to use for this customer"""


class DefaultPaymentMethod(TypedDict, total=False):
    """The default payment method details"""

    billing_id: Required[Annotated[Optional[str], PropertyInfo(alias="billingId")]]
    """The default payment method id"""

    card_expiry_month: Required[Annotated[Optional[float], PropertyInfo(alias="cardExpiryMonth")]]
    """The expiration month of the default payment method"""

    card_expiry_year: Required[Annotated[Optional[float], PropertyInfo(alias="cardExpiryYear")]]
    """The expiration year of the default payment method"""

    card_last4_digits: Required[Annotated[Optional[str], PropertyInfo(alias="cardLast4Digits")]]
    """The last 4 digits of the default payment method"""

    type: Required[Literal["CARD", "BANK", "CASH_APP"]]
    """The default payment method type"""


class Integration(TypedDict, total=False):
    """External billing or CRM integration link"""

    id: Required[str]
    """Integration details"""

    synced_entity_id: Required[Annotated[Optional[str], PropertyInfo(alias="syncedEntityId")]]
    """Synced entity id"""

    vendor_identifier: Required[
        Annotated[
            Literal[
                "AUTH0",
                "ZUORA",
                "STRIPE",
                "HUBSPOT",
                "AWS_MARKETPLACE",
                "SNOWFLAKE",
                "SALESFORCE",
                "BIG_QUERY",
                "OPEN_FGA",
                "APP_STORE",
            ],
            PropertyInfo(alias="vendorIdentifier"),
        ]
    ]
    """The vendor identifier of integration"""


class PassthroughStripeBillingAddress(TypedDict, total=False):
    """Physical address"""

    city: str
    """City name"""

    country: str
    """Country code or name"""

    line1: str
    """Street address line 1"""

    line2: str
    """Street address line 2"""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Postal or ZIP code"""

    state: str
    """State or province"""


class PassthroughStripeShippingAddress(TypedDict, total=False):
    """Physical address"""

    city: str
    """City name"""

    country: str
    """Country code or name"""

    line1: str
    """Street address line 1"""

    line2: str
    """Street address line 2"""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Postal or ZIP code"""

    state: str
    """State or province"""


class PassthroughStripeTaxID(TypedDict, total=False):
    """Tax identifier with type and value for customer tax exemptions."""

    type: Required[str]
    """The type of tax exemption identifier, such as VAT."""

    value: Required[str]
    """The actual tax identifier value"""


class PassthroughStripe(TypedDict, total=False):
    """Stripe-specific billing fields for the customer."""

    billing_address: Annotated[PassthroughStripeBillingAddress, PropertyInfo(alias="billingAddress")]
    """Physical address"""

    customer_name: Annotated[str, PropertyInfo(alias="customerName")]
    """Customer name"""

    invoice_custom_fields: Annotated[Dict[str, str], PropertyInfo(alias="invoiceCustomFields")]
    """Invoice custom fields"""

    metadata: Dict[str, str]
    """Additional metadata"""

    payment_method_id: Annotated[str, PropertyInfo(alias="paymentMethodId")]
    """Billing provider payment method id, attached to this customer"""

    shipping_address: Annotated[PassthroughStripeShippingAddress, PropertyInfo(alias="shippingAddress")]
    """Physical address"""

    tax_ids: Annotated[Iterable[PassthroughStripeTaxID], PropertyInfo(alias="taxIds")]
    """Tax IDs"""


class PassthroughZuoraBillingAddress(TypedDict, total=False):
    """Physical address"""

    city: str
    """City name"""

    country: str
    """Country code or name"""

    line1: str
    """Street address line 1"""

    line2: str
    """Street address line 2"""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Postal or ZIP code"""

    state: str
    """State or province"""


class PassthroughZuora(TypedDict, total=False):
    """Zuora-specific billing fields for the customer."""

    billing_address: Annotated[PassthroughZuoraBillingAddress, PropertyInfo(alias="billingAddress")]
    """Physical address"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """Customers selected currency"""

    metadata: Dict[str, str]
    """Additional metadata"""

    payment_method_id: Annotated[str, PropertyInfo(alias="paymentMethodId")]
    """Billing provider payment method id, attached to this customer"""


class Passthrough(TypedDict, total=False):
    """Vendor-specific billing passthrough fields."""

    stripe: PassthroughStripe
    """Stripe-specific billing fields for the customer."""

    zuora: PassthroughZuora
    """Zuora-specific billing fields for the customer."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = [
    "EntitlementUpdateParams",
    "UpdateFeatureEntitlementRequest",
    "UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration",
    "UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration",
    "UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration",
    "UpdateCreditEntitlementRequest",
]


class UpdateFeatureEntitlementRequest(TypedDict, total=False):
    plan_id: Required[Annotated[str, PropertyInfo(alias="planId")]]

    type: Required[Literal["FEATURE"]]
    """UpdateFeatureEntitlementRequest"""

    behavior: Literal["Increment", "Override"]
    """Entitlement behavior (Increment or Override)"""

    description: str
    """Description of the entitlement"""

    display_name_override: Annotated[str, PropertyInfo(alias="displayNameOverride")]
    """Override display name for the entitlement"""

    enum_values: Annotated[SequenceNotStr[str], PropertyInfo(alias="enumValues")]
    """Allowed enum values for the feature entitlement"""

    has_soft_limit: Annotated[bool, PropertyInfo(alias="hasSoftLimit")]
    """Whether the usage limit is a soft limit"""

    has_unlimited_usage: Annotated[bool, PropertyInfo(alias="hasUnlimitedUsage")]
    """Whether usage is unlimited"""

    hidden_from_widgets: Annotated[
        List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]], PropertyInfo(alias="hiddenFromWidgets")
    ]
    """Widget types where this entitlement is hidden"""

    is_custom: Annotated[bool, PropertyInfo(alias="isCustom")]
    """Whether this is a custom entitlement"""

    is_granted: Annotated[bool, PropertyInfo(alias="isGranted")]
    """Whether the entitlement is granted"""

    monthly_reset_period_configuration: Annotated[
        Optional[UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration],
        PropertyInfo(alias="monthlyResetPeriodConfiguration"),
    ]
    """Configuration for monthly reset period"""

    order: float
    """Display order of the entitlement"""

    reset_period: Annotated[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"], PropertyInfo(alias="resetPeriod")]
    """Period at which usage resets"""

    usage_limit: Annotated[Optional[int], PropertyInfo(alias="usageLimit")]
    """Maximum allowed usage for the feature"""

    weekly_reset_period_configuration: Annotated[
        Optional[UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration],
        PropertyInfo(alias="weeklyResetPeriodConfiguration"),
    ]
    """Configuration for weekly reset period"""

    yearly_reset_period_configuration: Annotated[
        Optional[UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration],
        PropertyInfo(alias="yearlyResetPeriodConfiguration"),
    ]
    """Configuration for yearly reset period"""


class UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration(TypedDict, total=False):
    """Configuration for monthly reset period"""

    according_to: Required[
        Annotated[Literal["SubscriptionStart", "StartOfTheMonth"], PropertyInfo(alias="accordingTo")]
    ]
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration(TypedDict, total=False):
    """Configuration for weekly reset period"""

    according_to: Required[
        Annotated[
            Literal[
                "SubscriptionStart",
                "EverySunday",
                "EveryMonday",
                "EveryTuesday",
                "EveryWednesday",
                "EveryThursday",
                "EveryFriday",
                "EverySaturday",
            ],
            PropertyInfo(alias="accordingTo"),
        ]
    ]
    """Reset anchor (SubscriptionStart or specific day)"""


class UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration(TypedDict, total=False):
    """Configuration for yearly reset period"""

    according_to: Required[Annotated[Literal["SubscriptionStart"], PropertyInfo(alias="accordingTo")]]
    """Reset anchor (SubscriptionStart)"""


class UpdateCreditEntitlementRequest(TypedDict, total=False):
    plan_id: Required[Annotated[str, PropertyInfo(alias="planId")]]

    type: Required[Literal["CREDIT"]]
    """UpdateCreditEntitlementRequest"""

    amount: float
    """Credit grant amount"""

    behavior: Literal["Increment", "Override"]
    """Entitlement behavior (Increment or Override)"""

    cadence: Literal["MONTH", "YEAR"]
    """Credit grant cadence (MONTH or YEAR)"""

    description: str
    """Description of the entitlement"""

    display_name_override: Annotated[str, PropertyInfo(alias="displayNameOverride")]
    """Override display name for the entitlement"""

    hidden_from_widgets: Annotated[
        List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]], PropertyInfo(alias="hiddenFromWidgets")
    ]
    """Widget types where this entitlement is hidden"""

    is_custom: Annotated[bool, PropertyInfo(alias="isCustom")]
    """Whether this is a custom entitlement"""

    is_granted: Annotated[bool, PropertyInfo(alias="isGranted")]
    """Whether the entitlement is granted"""

    order: float
    """Display order of the entitlement"""


EntitlementUpdateParams: TypeAlias = Union[UpdateFeatureEntitlementRequest, UpdateCreditEntitlementRequest]
